% Program processes images for training and validation
% Date Published: 11 Dec 2017
% Author: Jonathan Burnett - AIS and AirCTEMPS Labs
% Purpose: This program is intended to guide the user through a serious of
% prompts to collect pertinent information as it relates to validation and
% training images for calibrating whale data.  

% Usage: run the entire program and follow the prompts.  Each execution of
% this program is intended to run through a single SERIES of validation OR
% calibration/training images.  

% Program includes calibration files for P3 Pro, P4 (Non Pro) and P4 Pro,
% but also includes functionality to utilize custom calibrations or no
% calibration

%% Main program
%import images by flight and sighting

close all 
clear all 
%% global variables

%% camera information
cwd = (fileparts(mfilename('fullpath')))
% select proper calibration file
opts = {'Phantom 4 Video','Phantom 3 Pro Video','Phantom 4 Pro Video','Custom Calibration','No Calibration'}
qStr = 'Which camera was this image taken with?'
calfile = bttnChoiseDialog(opts,'Camera Model Selection', 1, qStr)
% Handle response
switch calfile
    case 1
        calfile = fullfile(cwd,'p4nonprocal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 3.61;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 2
        calfile = fullfile(cwd,'p3procal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 3.61;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 3
        calfile = fullfile(cwd,'p4procal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 8.80;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 4
        [FileName,PathName] = uigetfile('*.mat ','Select custom calibration with *.mat extention; note: Name in workspace must be "calibrationSession"');
        calfile = fullfile(PathName,char(FileName)); %concatenate filename and pathname
        load(calfile); %%when loaded this file must be named 'calibrationSession' in the workspace
        cameraParams = calibrationSession;
        %focal length
        prompt = {'Camera Focal Length in mm (e.g. 3.61 for P3/P4 and 8.8 mm for P4 Pro):'};
        dlg_title = 'Camera Focal Length';
        num_lines = 1;
        defaultans = {'3.61'};
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        cfl = str2double(answer{1});
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 5
        calfile = 'NA';
        %manual camera information entry
        %focal length
        prompt = {'Camera Focal Length in mm (e.g. 3.61 for P3/P4 and 8.8 mm for P4 Pro):'};
        dlg_title = 'Camera Focal Length';
        num_lines = 1;
        defaultans = {'3.61'};
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        cfl = str2double(answer{1});
        %default camera specs are for a 4K DJI phantom 3 pro video camera:
        %http://www.sony.net/Products/SC-HP/cx_news_archives/img/pdf/vol_70/imx117cqt.pdf
        %camera pixel pitch
        prompt = {'Camera sensor pixel pitch in mm (e.g. .00155 for P3/P4 and .0024 for P4 Pro):'};
        dlg_title = 'Input';
        num_lines = 1;
        defaultans = {'0.00155'}; %phantom 3/4 is 1.55 micrometers
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        pmm = str2double(answer{1});
        cameraParams = 'NA';
end 
%%

%%object information
    %object name
    prompt = {'Give the object a name (limit 15 characters, no spaces)'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'white_board'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    object_name = answer{1};
    
    %object length in mm
    prompt = {'Scaling object length in units if mm (e.g. 1000):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'1000'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    scalelength_mm = str2double(answer{1});
      
%     prompt = {'Training(T) or Validation(V): Enter T or V'};
%     dlg_title = 'Input';
%     num_lines = 1;
%     defaultans = {'T'};
%     answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    image_type = 'T' %remove validation option for now

%% whale image reading and measuring section
if image_type == 'T'
    message = sprintf(['Select a high, med and low altitude image containing the scaling object from both take off and landing']);
else
    message = sprintf(['Select at least five images from take off an landing to validate the model']);
end 
uiwait(msgbox(message));

[fileName,pathName,numfiles] = multifileopen(); % call multifileopen to open files for manipulation
whale_data = cell(numfiles(2),14); %#####CHANGE THE SECOND VALUE IF YOU CHANGE THE # of HEADINGS
wcnt = 1; %beginning of counter for # of images processed

%identificaiton information
%sighting #
prompt = {'What sighting does this image belong to (e.g. 1):'};
dlg_title = 'Sighting';
num_lines = 1;
defaultans = {'1'};
answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
sighting = int16(str2double(answer{1}));

%flight #
prompt = {'What flight does this belong to (e.g. 1):'};
dlg_title = 'Flight';
num_lines = 1;
defaultans = {'1'};
answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
flight = int16(str2double(answer{1}));

%date #
prompt = {'What date was this image taken mm/dd/yyyy (e.g. 07/07/2016: '};
dlg_title = 'Date';
num_lines = 1;
defaultans = {'07/17/2016'};
answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
fltdate = (answer{1});

% iterate through all images
for kk = 1:numfiles(2) 
    fullFileName = fullfile(pathName,char(fileName(kk))); %concatenate filename and pathname
    img = imread(fullFileName); %read image
    if cameraParams == 'NA' %if no calibration info skip
        img1 = img;
    else
        img1 = undistortImage(img, cameraParams); %undistorts images
    end 
    set(0,'units','pixels');  %Sets the units of your root object (screen) to pixels
    Pix_SS = get(0,'screensize');     %Obtains this pixel information


    %crops image
    %img1 = imrotate(img1,90,'bilinear');
    imshow(img1);
    message = sprintf(['Bound the scaling object with a square and right click and click "crop image" when you are done.']);
    uiwait(msgbox(message));
    I2 = imcrop(img1); 
    close all
    img_wiener = imageenhancer(I2); %enhances image
    %% Scaling Object Measurements section
    imshow(img_wiener);
    
    %measurement
    message = sprintf(['Click a point on each end of the scaling object']);
    test = msgbox(message);
    mpos = get(test,'OuterPosition'); 
    set(test,'OuterPosition',[mpos(1)+400,mpos(2:4)]);
    uwait=(test);
    hold on
    n = 2
    xlength=zeros(n,1);
    ylength=zeros(n,1);
    for ii = 1:n;
        [x1,y1]=ginput(1);
        plot(x1,y1,'--.r','MarkerSize',12);
        xlength(ii)=x1;
        ylength(ii)=y1;
    end 
    objectlength_p = sqrt((xlength(1)-xlength(2))^2+(ylength(1)-ylength(2))^2);
    plot(xlength,ylength,'LineWidth',2);
    dx=-5;dy=-5;
    text(xlength+dx,ylength+dy,'OL');       

    % plot measurements 
    % save image code from: http://stackoverflow.com/questions/1848176/how-do-i-save-a-plotted-image-and-maintain-the-original-image-size-in-matlab
    [pathstr,name,ext] = fileparts(fullFileName);
    plottitle = [' Flight: ' num2str(flight) ' File: ' name ' Image #: ' num2str(wcnt)]; 
    title(plottitle)
    f = getframe(gcf);              %# Capture the current window
    imgout = [fullFileName(1:end-4),'_measured.jpg']
    imwrite(f.cdata,imgout);  %# Save the frame data
    
    message = sprintf(['Press Enter twice to continue']);
    test = msgbox(message);
    pause
    close all
          
    
    %% Conversion from image coordinates to metric
    %Known altitude offset (bias)
    imshow(img)
    hold on
    prompt = {'Enter difference between take off height and object height (m); enter a negative value if object is higher than aircraft (e.g. 1.0):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'4.6'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    altbias = str2double(answer{1});
    
    %Record Altitude of whale
    imshow(img)
    hold on
    prompt = {'Altitude from bottom of image in meters(e.g. 35.0):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'35.0'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    balt = str2double(answer{1});
    hold off
    
    %GSD estimation in units of altitude (meters)
    adj_alt = balt+altbias;
    %pmm = cpwmm/cpwp %pixels size in mm/pixel
    GSD = (adj_alt/cfl)*pmm %from http://www.asprs.org/a/publications/pers/2004journal/march/2004_mar_297-300.pdf
    
    %'Flight,','Original_Filename,','Output_Image_Name,','Baro_Alt,','Adj_Baro_Alt,','GSD,','Focal_Length,','Cam_Pix_mm,','Object_Length','Scale_Length','Object_Name','Training_or_Validation'
    whale_data(wcnt,:) = {sighting,flight,fltdate,fullFileName,imgout,balt,adj_alt,GSD,cfl,pmm,objectlength_p,scalelength_mm,object_name,image_type}; %stores data and placeholders to be filled in later
    wcnt = wcnt+1; % global counter that is initialized at program start
    close all 
end 

% CSV file writing section
[pathstr,name,ext] = fileparts(fullFileName);
cd(pathstr);
fileout = ['scale_object_',image_type,'_flight_',num2str(flight),'_',date,'.csv'];
fileID = fopen(fileout,'w');
header = {'Sighting','Flight','Flight_Date','Original_Filename','Output_Image_Name','Baro_Alt','Adj_Baro_Alt','GSD','Focal_Length','Cam_Pix_mm','Object_Length_Pixels','Object_Length_mm','Object_Name','Validation_Training'};
formatSpec= '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n';
fprintf(fileID,formatSpec,header{1,:});
formatSpec= '%d,%d,%s,%s,%s,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%s,%s\n';
[nrows,ncols] = size(whale_data);
for row = 1:nrows;
    fprintf(fileID,formatSpec,whale_data{row,:});
end
fclose(fileID);
close all
