% Program processes images for measurement of morphometric attributes
% Date Published: 8 Jan 2019
% Author: Jonathan Burnett - AIS and AirCTEMPS Labs
% Purpose: This program is intended to guide the user through a serious of
% prompts to collect pertinent information as it relates to the UAS aerial
% survey flights of whales

% Usage: run the entire program and follow the prompts. 

% Program includes calibration files for P3 Pro, P4 (Non Pro) and P4 Pro,
% but also includes functionality to utilize custom calibrations or no
% calibration

%% Main program
%import images by flight and sighting
% 
 close all 
 clear all 

%% camera information
cwd = (fileparts(mfilename('fullpath')))
% select proper calibration file
opts = {'Phantom 4 Video','Phantom 3 Pro Video','Phantom 4 Pro Video','Custom Calibration','No Calibration'}
qStr = 'Which camera was this image taken with?'
calfile = bttnChoiseDialog(opts,'Camera Model Selection', 1, qStr)
% Handle response
switch calfile
    case 1
        calfile = fullfile(cwd,'p4nonprocal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 3.61;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 2
        calfile = fullfile(cwd,'p3procal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 3.61;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 3
        calfile = fullfile(cwd,'p4procal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 8.80;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 4
        [FileName,PathName] = uigetfile('*.mat ','Select custom calibration with *.mat extention; note: Name in workspace must be "calibrationSession"');
        calfile = fullfile(PathName,char(FileName)); %concatenate filename and pathname
        load(calfile); %%when loaded this file must be named 'calibrationSession' in the workspace
        cameraParams = calibrationSession;
        %focal length
        prompt = {'Camera Focal Length in mm (e.g. 3.61 for P3/P4 and 8.8 mm for P4 Pro):'};
        dlg_title = 'Camera Focal Length';
        num_lines = 1;
        defaultans = {'3.61'};
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        cfl = str2double(answer{1});
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 5
        calfile = 'NA';
        %manual camera information entry
        %focal length
        prompt = {'Camera Focal Length in mm (e.g. 3.61 for P3/P4 and 8.8 mm for P4 Pro):'};
        dlg_title = 'Camera Focal Length';
        num_lines = 1;
        defaultans = {'3.61'};
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        cfl = str2double(answer{1});
        %default camera specs are for a 4K DJI phantom 3 pro video camera:
        %http://www.sony.net/Products/SC-HP/cx_news_archives/img/pdf/vol_70/imx117cqt.pdf
        %camera pixel pitch
        prompt = {'Camera sensor pixel pitch in mm (e.g. .00155 for P3/P4 and .0033 for P4 Pro):'};
        dlg_title = 'Input';
        num_lines = 1;
        defaultans = {'0.00155'}; %phantom 3/4 is 1.55 micrometers
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        pmm = str2double(answer{1});
        cameraParams = 'NA';
end 
          

%% whale image reading and measuring section
%select files
[fileName,pathName,numfiles] = multifileopen(); % call multifileopen to open files for manipulation
% numfiles = 1 %used for debugging purposes
whale_data = cell(numfiles(2),23);
wcnt = 1;
for kk = 1:numfiles(2) %iterate through all images
    fullFileName = fullfile(pathName,char(fileName(kk))); %concatenate filename and pathname
    img = imread(fullFileName); %read image
    
    if cameraParams == 'NA' %if no calibration info skip
        img1 = img;
    else
        img1 = undistortImage(img, cameraParams); %undistorts images
    end 
    %Sets the units of your root object (screen) to pixels
    set(0,'units','pixels')  
    %Obtains this pixel information
    Pix_SS = get(0,'screensize')
    imshow(img1);
    
    %SIGHTING #
    prompt = {'What sighting does this belong to (e.g. 1):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'1'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    sighting = str2double(answer{1});
    
    %flight #
    %flight = 1; %debugging
    prompt = {'What flight does this belong to (e.g. 1):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'1'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    flight = str2double(answer{1});
    
    %Date#
    prompt = {'Enter the date the whale was imaged in MM/DD/YYYY (e.g. "07/30/2016"):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'01/01/1980'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    fltdate = (answer{1});
    
    %whale ID#
    prompt = {'Provide a whale identification; essential for images with more than one whale (e.g. "A"):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'A'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    whaleID = (answer{1});
    
      %% Conversion from image coordinates to metric
    %Known altitude offset (bias)
    imshow(img)
    hold on
    prompt = {'Enter difference between take off height and actual water level in meters (e.g. 2.74):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'2.74'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    altbias = str2double(answer{1});
    
    %Record Altitude of whale
    imshow(img)
    hold on
    prompt = {'Altitude (barometer) from bottom of image in meters(e.g. 35.0):'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'35.0'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    balt = str2double(answer{1});
    
    %GSD estimation in units of altitude (meters)
    adj_alt = balt+altbias;
    GSD = (adj_alt/cfl)*pmm %from http://www.asprs.org/a/publications/pers/2004journal/march/2004_mar_297-300.pdf
    
    %crops image
    %img1 = imrotate(img1,90,'bilinear');
    imshow(img1);
    message = sprintf(['Bound the whale with a square and right click and click "crop image" when you are done.']);
    uiwait(msgbox(message));
    I2 = imcrop(img1);
    
    %user guided rotation
    %     imshow(I2);
    msg = 'Click 1 point on the nose on the whale and 1 point on the tail end of the whale to create an horizontal axis for measurement';
    num = 2
    label = 'X'
    [xlength,ylength,tailwidth_p] = imageplot1(I2,msg,label,num);
    lmdl = LinearModel.fit(xlength,ylength)
    rot_angle = radtodeg(atan(lmdl.Coefficients.Estimate(2))); %rotate to make whale perfectly horizontal
    I2 = imrotate(I2,rot_angle,'bilinear');
    img_wiener = imageenhancer(I2); %enhances image
    
    
    %% Whale measurements section
    
    %%tail width measurement
    msg = 'Click 1 point on the left side of the whales fluke and 1 point on the right side of the whales fluke';
    label = 'FW';
    num = 2; %number of point measurements
    [xtail,ytail,tailwidth_p] = imageplot1(img_wiener,msg,label,num);
    
    %length measurements
    msg = 'Click 1 point on the nose on the whale and 1 point on the tail, starting with the nose';
    label = 'WL';
    num = 2;
    [xlength,ylength,bodylength_p] = imageplot1(img_wiener,msg,label,num);
    
    %make xlength position 1 the nose based on where the tail is (just in case the user clicks order wrong
    [M,I] = min(abs(xlength-xtail(1)));
    if I == 1 %changes position if the tail is actually in xlength position 1
        xlength = [xlength(2);xlength(1)];
    end 
    %width measurements
    %side 1 of whale
    perc_m = [0.15,.20,.25,.30,.35,.40,.45,.50,.55,.60,.65];
    perc_m2 = [0.20,.30,.40,.50,.60]
    msg = 'On the most visible side of whale, outline the whale profile by clicking\r where the 11 vertical lines intersect the body';
    num = length(perc_m);
    [x1l,y1l,r2s1,sa1,perc_x1,perc_y1,perc_w1] = imageplot2(img_wiener,msg,num,perc_m,perc_m2,bodylength_p,xlength,ylength,0,0)
    %side 2 of whale
    msg = 'On the other side opposite side of the whale, outline the whale profile by clicking\r where the 11 vertical lines intersect the body';
    num = length(perc_m);
    [x1r,y1r,r2s2,sa2,perc_x2,perc_y2,perc_w2] = imageplot2(img_wiener,msg,num,perc_m,perc_m2,bodylength_p,xlength,ylength,x1l,y1l)
    
    %%manual body width measurement
    msg = 'Click 1 point on both sides of the whale body at the widest section';
    num = 2
    label = 'MW';
    [xside_m,yside_m,bodywidth2_p] = imageplot1(img_wiener,msg,label,num);
    
    
    
    
    %%calculate optimal widest point (closest distance to line)
    [idx1, d1] = nearestpoint(yside_m(1),y1r,'nearest');
    [idx2, d2] = nearestpoint(yside_m(2),y1r,'nearest');
    try
        if (d1 < d2); %d1 and thusly side_m(1) is associated with the y1r and x1r coordinates
            [xside_o(1),yside_o(1)] = shortest_distance(xside_m(1),yside_m(1),x1r,y1r);
            [xside_o(2),yside_o(2)] = shortest_distance(xside_m(2),yside_m(2),x1l,y1l);
        elseif (d1 > d2);
            [xside_o(1),yside_o(1)] = shortest_distance(xside_m(1),yside_m(1),x1l,y1l);
            [xside_o(2),yside_o(2)] = shortest_distance(xside_m(2),yside_m(2),x1r,y1r);
        end 
    
    catch ME
        msg = 'Optimal widest point could not be estimated. This is likely due to the parabola for one or both sides being concave and not convex. Remeasure whale(s)';
        msgID = ME.identifier;
        causeException = MException(msgID,msg);
        ME = addCause(ME,causeException);
        %xside_o = {'0','0'};
        %yside_o = {'0','0'};
        throw(ME)
    end 
    
    bodywidth3_p = distancequation(xside_o(1),xside_o(2),yside_o(1),yside_o(2));

    %width calculations
    perc_l = perc_w1+perc_w2
    
    %surface area calculations
    sa_total = sa1+sa2
    
    % plot measurements 
    close all %closes any open windows or prompts
    imshow(img_wiener);
    axis xy;
    set(gcf,'Units','normalized','Position',[0,0,1,1]);  %# full screen figure
    hold on;

    plot(xlength,ylength,'LineWidth',8); %length
    plot(xtail,ytail,'LineWidth',8); %tail width
    plot(x1r,y1r,'LineWidth',8); %side 1
    plot(x1l,y1l,'LineWidth',8); %side 2
    plot(xside_o,yside_o,'-mX','LineWidth',8,'MarkerSize',18); %optimal width
    plot(xside_m,yside_m,'-mX','LineWidth',8,'MarkerSize',18); %manual width
    for ii = 1:length(perc_x2); %shows gridlines 
        plot([perc_x2(ii),perc_x2(ii)],ylim,'-.y','MarkerSize',18);
            dx = 0; dy = -20; % displacement so the text does not overlay the data points
        text(perc_x2(ii)+dx,perc_y2(ii)+dy,num2str(perc_m2(ii)),'FontSize',32,'Color','y'); 
        plot(perc_x2(ii),perc_y2(ii),'mX','MarkerSize',16); %optimal width
    end
    dx = 0; dy = -5; % displacement so the text does not overlay the data point
    text(xside_m(1)+dx,yside_m(1)+25,'OW','FontSize',32,'Color','y');
    text(xtail+dx,ytail+dy,'FW','FontSize',32,'Color','y'); 
    text(xlength(2)+5,ylength(2)+dy,'WL','FontSize',32,'Color','y'); 
    text(xlength(1)-60,ylength(1)+dy,'WL','FontSize',32,'Color','y'); 
    dat =  {GSD;bodylength_p;tailwidth_p;sa_total;bodywidth3_p};

    uicontrol('style','text', 'units','normalized','FontSize',24, ...
    'position',[.81 .05 .15 .04], 'string',sprintf('WL (p): %10.2f', dat{2}))
    uicontrol('style','text', 'units','normalized','FontSize',24, ...
    'position',[.81 .09 .15 .04], 'string',sprintf('GSD (m): %10.2f', dat{1}))
    uicontrol('style','text', 'units','normalized', 'FontSize',24,...
    'position',[.81 .13 .15 .04], 'string',sprintf('OW (p): %10.2f', dat{5}))
    uicontrol('style','text', 'units','normalized','FontSize',24, ...
    'position',[.81 .17 .165 .04], 'string',sprintf('SA (p^2): %10.2f', dat{4}))
    uicontrol('style','text', 'units','normalized','FontSize',24, ...
    'position',[.81 .21 .15 .04], 'string',sprintf('FW (p): %10.2f', dat{3}))
    
    % save image code from: http://stackoverflow.com/questions/1848176/how-do-i-save-a-plotted-image-and-maintain-the-original-image-size-in-matlab
    [pathstr,name,ext] = fileparts(fullFileName);
    plottitle = ['Sighting: ' num2str(sighting) ' Flight: ' num2str(flight) ' Whale: ' (whaleID) ' File: ' name ' Image #: ' num2str(wcnt)]; 
    title(plottitle,'FontSize',14)
    f = getframe(gcf);              %# Capture the current window
    imgout = [fullFileName(1:end-4),'_measured.jpg']
    imwrite(f.cdata,imgout);  %# Save the frame data
    message = sprintf(['Press Enter 2x to continue']);
    test = msgbox(message);
    pause
    close all
          
    %'Sighting,','Flight,','Original_Filename,','Output_Image_Name,','Baro_Alt,','Adj_Baro_Alt,','GSD,','Focal_Length,','Cam_Pix_mm,','R2_S1','R2_S2','SA','WL,','FW','MW,','OW,','Pcnt20Wd,','Pcnt30Wd,','Pcnt40Wd,','Pcnt50Wd,','Pcnt60Wd,'
    whale_data(wcnt,:) = {sighting,flight,whaleID,fltdate,fullFileName,imgout,balt,adj_alt,GSD,cfl,pmm,r2s1,r2s2,sa_total,bodylength_p,tailwidth_p,bodywidth2_p,bodywidth3_p,perc_l(1),perc_l(2),perc_l(3),perc_l(4),perc_l(5)}; %stores data and placeholders to be filled in later
    wcnt = wcnt+1; % global counter that is initialized at program start
    kk = kk+1;
end 

[pathstr,name,ext] = fileparts(fullFileName);
cd(pathstr);
isunique = 1
filenum = 1
while isunique > 0 %loops through until it finds a unique file suffix
    fileout = ['whale_measurements_',char(datetime('today')),'_',char(string(filenum)),'.csv'];
    if exist(fileout) == 0;
        fileID = fopen(fileout,'wt');
        isunique=0
    else
        filenum = str2num(fileout(end-4:end-4))+1;
        fileout = ['whale_measurements_',char(datetime('today')),'_',char(string(filenum)),'.csv'];
    end 
end 
    
[nrows,ncols] = size(whale_data);
header = {'Sighting','Flight','Whale','Flight_Date','Original_Filename','Output_Image_Name','Baro_Alt','Adj_Baro_Alt','GSD','Focal_Length','Cam_Pix_mm','R2_S1','R2_S2','SA','WL','FW','MW','OW','Pcnt20Wd','Pcnt30Wd','Pcnt40Wd','Pcnt50Wd','Pcnt60Wd'};
formatSpec= '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n';
fprintf(fileID,formatSpec,header{1,:});
formatSpec= '%d,%d,%s,%s,%s,%s,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f,%0.4f\n';
for row = 1:nrows;
    fprintf(fileID,formatSpec,whale_data{row,:});
end
fclose(fileID);
